#ifndef RAM_DISK_H
#define RAM_DISK_H

//    This file is part of JustMotion.

//    JustMotion is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    JustMotion is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

#include "common.h"

#define TMP_DIR "#TMP"

mode_t dirDefaultMode();
mode_t fileDefaultMode();
mode_t symmDefaultMode();

class RamDisk : public QObject
{
    Q_OBJECT

private:

    static RamDisk                *instance;
    static QString                 mnt;
    static QStringList             flushHist;
    static QStringList             bondDirs;
    static QHash<QString, file_t*> data;
    static fuse_operations         fs_oper;
    static quint64                 rootMtime;
    static quint64                 rootCtime;
    static quint64                 inoSeed;
    static quint64                 duMax;
    static quint64                 du;
    static QRecursiveMutex         mutex;

    static QString getFileName(const QString &basePath, const QString &fullPath);
    static quint32 getMode(const QString &path);
    static bool    dirEmpty(const QString &path, int &ret);
    static void    fillStatTimes(struct stat *info, quint64 cTimeSec, quint64 mTimeSec);
    static void    rePath(const QString &src, const QString &dst);
    static void    lock();
    static void    unlock();
    static int     calcNumOfBlocks(quint64 len, bool countZero = true);

private slots:

    void exec();

public:

    static bool  isFull();
    static int   getattr(const char *path, struct stat *info, struct fuse_file_info *fi);
    static int   readlink(const char *path, char *buff, size_t len);
    static int   readdir(const char *path, void *buff, fuse_fill_dir_t filler, off_t offset, struct fuse_file_info *fi, enum fuse_readdir_flags flags);
    static int   mkdir(const char *path, mode_t mode);
    static int   symlink(const char *src, const char *dst);
    static int   unlink(const char *path);
    static int   rmdir(const char *path);
    static int   chmod(const char *path, mode_t mode, struct fuse_file_info *fi);
    static int   chown(const char *path, uid_t uid, gid_t gid, struct fuse_file_info *fi);
    static int   flush(const char *path, struct fuse_file_info *fi);
    static int   release(const char *path, struct fuse_file_info *fi);
    static int   rename(const char *src, const char *dst, unsigned int flags);
    static int   truncate(const char *path, off_t size, struct fuse_file_info *fi);
    static int   open(const char *path, struct fuse_file_info *fi);
    static int   create(const char *path, mode_t mode, struct fuse_file_info *fi);
    static int   read(const char *path, char *buff, size_t len, off_t offs, struct fuse_file_info *fi);
    static int   write(const char *path, const char *buff, size_t len, off_t offs, struct fuse_file_info *fi);
    static int   statfs(const char *path, struct statvfs *stbuff);
    static int   utimens(const char *path, const struct timespec newTimes[2], struct fuse_file_info *fi);
    static int   fallocate(const char *path, int mode, off_t offs, off_t len, struct fuse_file_info *fi);
    static void *init(struct fuse_conn_info *conn, struct fuse_config *cfg);
    static void  destroy(void *privateData);
    static void  addCam(const QString &name);
    static void  rmCam(const QString &name);

    explicit RamDisk(QThread *thr, quint64 maxSize, const QString &mntPnt, QObject *parent = nullptr);

signals:

    void watchedFileFlushed(const QString &path);
};

#endif // RAM_DISK_H
